import { Container } from '../utils/container';
import { CommandHandler } from './CommandHandler';
import { SubscriberMetadata, SubscribersMetadata } from '../metadata/MetadataStorage';
import type { Class } from 'type-fest';
export interface CommandRunnerConfig {
    container?: Container;
}
/**
 * Runs commands
 */
export declare class CommandRunner {
    protected container: Container;
    protected handlers: Map<Class<CommandHandler>, CommandHandler>;
    protected subscribers: Map<Class<CommandHandler>, SubscriberMetadata>;
    constructor(config?: CommandRunnerConfig);
    /**
     * Executes commands.
     */
    run<T extends CommandHandler>(Handler: Class<T>, event: Parameters<T['handle']>['0']): Promise<ReturnType<T['handle']>>;
    /**
     * Serially executes `@BeforeCommand(Handler)` or `@AfterCommand(Handler)` decorated methods for the given `Handler`.
     */
    protected emit<T extends CommandHandler>(type: keyof SubscribersMetadata, HandlerClass: Class<T>, args: any[]): Promise<void>;
}
//# sourceMappingURL=CommandRunner.d.ts.map