"use strict";
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    function adopt(value) { return value instanceof P ? value : new P(function (resolve) { resolve(value); }); }
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : adopt(result.value).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.CommandRunner = void 0;
const container_1 = require("../utils/container");
const CommandHandler_1 = require("./CommandHandler");
const MetadataStorage_1 = require("../metadata/MetadataStorage");
const isPromise_1 = require("../utils/isPromise");
/**
 * Runs commands
 */
class CommandRunner {
    constructor(config = {}) {
        this.handlers = new Map();
        this.subscribers = new Map();
        this.container = config.container || new container_1.DefaultContainer();
    }
    /**
     * Executes commands.
     */
    run(Handler, event) {
        return __awaiter(this, void 0, void 0, function* () {
            const handlerOrPromise = this.container.get(Handler);
            const handler = isPromise_1.isPromise(handlerOrPromise)
                ? yield handlerOrPromise
                : handlerOrPromise;
            if (handler instanceof CommandHandler_1.AbstractCommandHandler) {
                handler.setRunner(this);
            }
            if (!handler) {
                throw new Error('Invalid handler');
            }
            yield this.emit('middleware', Handler, [event]);
            yield this.emit('before', Handler, [event]);
            const result = yield handler.handle(event);
            yield this.emit('after', Handler, [event, result]);
            return result;
        });
    }
    /**
     * Serially executes `@BeforeCommand(Handler)` or `@AfterCommand(Handler)` decorated methods for the given `Handler`.
     */
    emit(type, HandlerClass, args) {
        return __awaiter(this, void 0, void 0, function* () {
            if (MetadataStorage_1.storage.metadata.subscribers[type].has(HandlerClass)) {
                const subscribers = MetadataStorage_1.storage.metadata.subscribers[type].get(HandlerClass);
                args.push(this);
                for (const meta of subscribers) {
                    const subscriber = this.container.get(meta.SubscriberClass);
                    yield subscriber[meta.method].apply(subscriber, args);
                }
            }
        });
    }
}
exports.CommandRunner = CommandRunner;
//# sourceMappingURL=CommandRunner.js.map