"use strict";
var __decorate = (this && this.__decorate) || function (decorators, target, key, desc) {
    var c = arguments.length, r = c < 3 ? target : desc === null ? desc = Object.getOwnPropertyDescriptor(target, key) : desc, d;
    if (typeof Reflect === "object" && typeof Reflect.decorate === "function") r = Reflect.decorate(decorators, target, key, desc);
    else for (var i = decorators.length - 1; i >= 0; i--) if (d = decorators[i]) r = (c < 3 ? d(r) : c > 3 ? d(target, key, r) : d(target, key)) || r;
    return c > 3 && r && Object.defineProperty(target, key, r), r;
};
var __metadata = (this && this.__metadata) || function (k, v) {
    if (typeof Reflect === "object" && typeof Reflect.metadata === "function") return Reflect.metadata(k, v);
};
var __param = (this && this.__param) || function (paramIndex, decorator) {
    return function (target, key) { decorator(target, key, paramIndex); }
};
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    function adopt(value) { return value instanceof P ? value : new P(function (resolve) { resolve(value); }); }
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : adopt(result.value).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
Object.defineProperty(exports, "__esModule", { value: true });
require("reflect-metadata");
const inversify_1 = require("inversify");
const MetadataStorage_1 = require("../metadata/MetadataStorage");
const CommandRunner_1 = require("./CommandRunner");
const decorators_1 = require("../decorators");
const initState = () => ({
    calls: 0,
    args: [],
});
const updateState = (obj, args) => {
    obj.calls++;
    obj.args.push([...args]);
};
describe('CommandRunner', () => {
    beforeEach(() => {
        MetadataStorage_1.storage.clear();
    });
    test('simple command', () => __awaiter(void 0, void 0, void 0, function* () {
        class Simple {
            constructor() {
                updateState(Simple.state.constructor, arguments);
            }
            handle(event) {
                return __awaiter(this, arguments, void 0, function* () {
                    updateState(Simple.state.handle, arguments);
                    return event;
                });
            }
        }
        Simple.state = {
            constructor: initState(),
            handle: initState(),
        };
        const runner = new CommandRunner_1.CommandRunner();
        const event = {};
        expect(yield runner.run(Simple, event)).toBe(event);
        expect(Simple.state.constructor.calls).toBe(1);
        expect(Simple.state.constructor.args).toStrictEqual([[]]);
        expect(Simple.state.handle.calls).toBe(1);
        expect(Simple.state.handle.args).toStrictEqual([[event]]);
        // make sure command was constructed once & handle was called a second time
        const event2 = {};
        expect(yield runner.run(Simple, event2)).toBe(event2);
        expect(Simple.state.constructor.calls).toBe(1);
        expect(Simple.state.handle.calls).toBe(2);
    }));
    test('single @AfterCommand() subscriber', () => __awaiter(void 0, void 0, void 0, function* () {
        class Command {
            constructor() {
                updateState(Command.state.constructor, arguments);
            }
            handle(event) {
                return __awaiter(this, arguments, void 0, function* () {
                    updateState(Command.state.handle, arguments);
                    return event.result;
                });
            }
        }
        Command.state = {
            constructor: initState(),
            handle: initState(),
        };
        class CommandSubscriber {
            constructor() {
                updateState(CommandSubscriber.state.constructor, arguments);
            }
            onCommand(_event, _result, _runner) {
                return __awaiter(this, arguments, void 0, function* () {
                    updateState(CommandSubscriber.state.onCommand, arguments);
                });
            }
        }
        CommandSubscriber.state = {
            constructor: initState(),
            onCommand: initState(),
        };
        __decorate([
            decorators_1.AfterCommand(Command),
            __metadata("design:type", Function),
            __metadata("design:paramtypes", [Object, Boolean, CommandRunner_1.CommandRunner]),
            __metadata("design:returntype", Promise)
        ], CommandSubscriber.prototype, "onCommand", null);
        const runner = new CommandRunner_1.CommandRunner();
        const result = { foo: 'bar' };
        const event = { result };
        expect(yield runner.run(Command, event)).toBe(result);
        expect(Command.state.constructor.calls).toBe(1);
        expect(Command.state.constructor.args).toEqual([[]]);
        expect(Command.state.handle.calls).toBe(1);
        expect(Command.state.handle.args).toStrictEqual([[event]]);
        expect(CommandSubscriber.state.constructor.calls).toBe(1);
        expect(CommandSubscriber.state.constructor.args).toEqual([[]]);
        expect(CommandSubscriber.state.onCommand.calls).toBe(1);
        expect(CommandSubscriber.state.onCommand.args).toStrictEqual([
            [event, result, runner],
        ]);
    }));
    test('multiple @AfterCommand() subscribers with default priority', () => __awaiter(void 0, void 0, void 0, function* () {
        class Command {
            constructor() {
                updateState(Command.state.constructor, arguments);
            }
            handle(event) {
                return __awaiter(this, arguments, void 0, function* () {
                    updateState(Command.state.handle, arguments);
                    return event;
                });
            }
        }
        Command.state = {
            constructor: initState(),
            handle: initState(),
        };
        class CommandSubscriber1 {
            constructor() {
                updateState(CommandSubscriber1.state.constructor, arguments);
            }
            onCommand() {
                return __awaiter(this, arguments, void 0, function* () {
                    updateState(CommandSubscriber1.state.onCommand, arguments);
                });
            }
        }
        CommandSubscriber1.state = {
            constructor: initState(),
            onCommand: initState(),
        };
        __decorate([
            decorators_1.AfterCommand(Command),
            __metadata("design:type", Function),
            __metadata("design:paramtypes", []),
            __metadata("design:returntype", Promise)
        ], CommandSubscriber1.prototype, "onCommand", null);
        class CommandSubscriber2 {
            constructor() {
                updateState(CommandSubscriber2.state.constructor, arguments);
            }
            onCommand() {
                return __awaiter(this, arguments, void 0, function* () {
                    updateState(CommandSubscriber2.state.onCommand, arguments);
                });
            }
        }
        CommandSubscriber2.state = {
            constructor: initState(),
            onCommand: initState(),
        };
        __decorate([
            decorators_1.AfterCommand(Command),
            __metadata("design:type", Function),
            __metadata("design:paramtypes", []),
            __metadata("design:returntype", Promise)
        ], CommandSubscriber2.prototype, "onCommand", null);
        const runner = new CommandRunner_1.CommandRunner();
        const event = {};
        expect(yield runner.run(Command, event)).toBe(event);
        expect(Command.state.constructor.calls).toBe(1);
        expect(Command.state.constructor.args).toEqual([[]]);
        expect(Command.state.handle.calls).toBe(1);
        expect(Command.state.handle.args).toStrictEqual([[event]]);
        expect(CommandSubscriber1.state.constructor.calls).toBe(1);
        expect(CommandSubscriber1.state.constructor.args).toEqual([[]]);
        expect(CommandSubscriber1.state.onCommand.calls).toBe(1);
        expect(CommandSubscriber1.state.onCommand.args).toStrictEqual([
            [event, event, runner],
        ]);
        expect(CommandSubscriber2.state.constructor.calls).toBe(1);
        expect(CommandSubscriber2.state.constructor.args).toEqual([[]]);
        expect(CommandSubscriber2.state.onCommand.calls).toBe(1);
        expect(CommandSubscriber2.state.onCommand.args).toStrictEqual([
            [event, event, runner],
        ]);
    }));
    test('@Use() gets called before command handler', () => __awaiter(void 0, void 0, void 0, function* () {
        class ValidateCommand {
            use(_event) {
                return __awaiter(this, void 0, void 0, function* () {
                    throw new Error('Invalid Event');
                });
            }
        }
        class MyCommand {
            handle(event) {
                return __awaiter(this, void 0, void 0, function* () {
                    return event;
                });
            }
        }
        __decorate([
            decorators_1.Use(ValidateCommand),
            __metadata("design:type", Function),
            __metadata("design:paramtypes", [Object]),
            __metadata("design:returntype", Promise)
        ], MyCommand.prototype, "handle", null);
        const runner = new CommandRunner_1.CommandRunner();
        yield expect(runner.run(MyCommand, {})).rejects.toStrictEqual(new Error('Invalid Event'));
    }));
    test('multiple @AfterCommand() subscribers with priority', () => __awaiter(void 0, void 0, void 0, function* () {
        const calls = [];
        class Command {
            handle(event) {
                return __awaiter(this, void 0, void 0, function* () {
                    return event;
                });
            }
        }
        // @ts-ignore
        class CommandSubscriber1 {
            onCommand() {
                return __awaiter(this, void 0, void 0, function* () {
                    calls.push('CommandSubscriber1');
                });
            }
        }
        __decorate([
            decorators_1.AfterCommand(Command, { priority: -1 }),
            __metadata("design:type", Function),
            __metadata("design:paramtypes", []),
            __metadata("design:returntype", Promise)
        ], CommandSubscriber1.prototype, "onCommand", null);
        // @ts-ignore
        class CommandSubscriber2 {
            onCommand() {
                return __awaiter(this, void 0, void 0, function* () {
                    calls.push('CommandSubscriber2');
                });
            }
        }
        __decorate([
            decorators_1.AfterCommand(Command) // priority 0 (default)
            ,
            __metadata("design:type", Function),
            __metadata("design:paramtypes", []),
            __metadata("design:returntype", Promise)
        ], CommandSubscriber2.prototype, "onCommand", null);
        // @ts-ignore
        class CommandSubscriber3 {
            onCommand() {
                return __awaiter(this, void 0, void 0, function* () {
                    calls.push('CommandSubscriber3');
                });
            }
        }
        __decorate([
            decorators_1.AfterCommand(Command, { priority: 1 }),
            __metadata("design:type", Function),
            __metadata("design:paramtypes", []),
            __metadata("design:returntype", Promise)
        ], CommandSubscriber3.prototype, "onCommand", null);
        const runner = new CommandRunner_1.CommandRunner();
        yield runner.run(Command, {});
        expect(calls).toStrictEqual([
            'CommandSubscriber3',
            'CommandSubscriber2',
            'CommandSubscriber1',
        ]);
    }));
    test('@BeforeCommand() and @AfterCommand() with multiple subscribers', () => __awaiter(void 0, void 0, void 0, function* () {
        const calls = [];
        class User {
        }
        class CreateUser {
            handle(fields) {
                return __awaiter(this, arguments, void 0, function* () {
                    calls.push({ name: 'CreateUser', args: [...arguments] });
                    return Object.assign(new User(), Object.assign({}, fields));
                });
            }
        }
        class UserSubscribe {
            handle(email) {
                return __awaiter(this, arguments, void 0, function* () {
                    calls.push({ name: 'UserSubscribe', args: [...arguments] });
                    return email;
                });
            }
        }
        class SendEmail {
            handle(email) {
                return __awaiter(this, arguments, void 0, function* () {
                    calls.push({ name: 'SendEmail', args: [...arguments] });
                    return email;
                });
            }
        }
        // @ts-ignore
        class CreateUserSubscriber {
            lowercase(fields) {
                return __awaiter(this, arguments, void 0, function* () {
                    const args = [...arguments];
                    args[0] = Object.assign({}, args[0]); // clone args since we're mutating
                    calls.push({ name: 'CreateUserSubscriber.lowercase', args });
                    fields.email = fields.email.toLowerCase();
                });
            }
            onCreateUser(_fields, user, runner) {
                return __awaiter(this, arguments, void 0, function* () {
                    calls.push({
                        name: 'CreateUserSubscriber.onCreateUser',
                        args: [...arguments],
                    });
                    yield runner.run(UserSubscribe, user.email);
                });
            }
        }
        __decorate([
            decorators_1.BeforeCommand(CreateUser),
            __metadata("design:type", Function),
            __metadata("design:paramtypes", [Object]),
            __metadata("design:returntype", Promise)
        ], CreateUserSubscriber.prototype, "lowercase", null);
        __decorate([
            decorators_1.AfterCommand(CreateUser),
            __metadata("design:type", Function),
            __metadata("design:paramtypes", [Object, User,
                CommandRunner_1.CommandRunner]),
            __metadata("design:returntype", Promise)
        ], CreateUserSubscriber.prototype, "onCreateUser", null);
        // @ts-ignore
        class OnUserSubscribe {
            onUserSubscribe(_event, result, runner) {
                return __awaiter(this, arguments, void 0, function* () {
                    calls.push({ name: 'OnUserSubscribe', args: [...arguments] });
                    yield runner.run(SendEmail, result);
                });
            }
        }
        __decorate([
            decorators_1.AfterCommand(UserSubscribe),
            __metadata("design:type", Function),
            __metadata("design:paramtypes", [String, String, CommandRunner_1.CommandRunner]),
            __metadata("design:returntype", Promise)
        ], OnUserSubscribe.prototype, "onUserSubscribe", null);
        const runner = new CommandRunner_1.CommandRunner();
        const user = yield runner.run(CreateUser, { email: 'JOHN@DOE.COM' });
        expect(user).toBeInstanceOf(User);
        expect(user).toStrictEqual(Object.assign(new User(), { email: 'john@doe.com' }));
        expect(calls).toStrictEqual([
            {
                name: 'CreateUserSubscriber.lowercase',
                args: [{ email: 'JOHN@DOE.COM' }, runner],
            },
            {
                name: 'CreateUser',
                args: [{ email: 'john@doe.com' }],
            },
            {
                name: 'CreateUserSubscriber.onCreateUser',
                args: [{ email: 'john@doe.com' }, user, runner],
            },
            {
                name: 'UserSubscribe',
                args: ['john@doe.com'],
            },
            {
                name: 'OnUserSubscribe',
                args: ['john@doe.com', 'john@doe.com', runner],
            },
            {
                name: 'SendEmail',
                args: ['john@doe.com'],
            },
        ]);
    }));
    it('uses inversify Container for DI', () => __awaiter(void 0, void 0, void 0, function* () {
        class User {
        }
        const expected = new User();
        expected.name = 'John Doe';
        let UserRepository = class UserRepository {
            getUser() {
                return __awaiter(this, void 0, void 0, function* () {
                    return expected;
                });
            }
        };
        UserRepository = __decorate([
            inversify_1.injectable()
        ], UserRepository);
        const log = jest.fn();
        let Logger = class Logger {
            log(msg) {
                return __awaiter(this, void 0, void 0, function* () {
                    log(msg);
                });
            }
        };
        Logger = __decorate([
            inversify_1.injectable()
        ], Logger);
        let GetUserCommand = class GetUserCommand {
            constructor(userRepository) {
                this.userRepository = userRepository;
            }
            handle() {
                return __awaiter(this, void 0, void 0, function* () {
                    return this.userRepository.getUser();
                });
            }
        };
        GetUserCommand = __decorate([
            inversify_1.injectable(),
            __param(0, inversify_1.inject(UserRepository)),
            __metadata("design:paramtypes", [UserRepository])
        ], GetUserCommand);
        let OnGetUser = 
        // @ts-ignore
        class OnGetUser {
            constructor(logger) {
                this.logger = logger;
            }
            onGetUser(_event, user) {
                return __awaiter(this, void 0, void 0, function* () {
                    this.logger.log(`Hello, ${user.name}`);
                });
            }
        };
        __decorate([
            decorators_1.AfterCommand(GetUserCommand),
            __metadata("design:type", Function),
            __metadata("design:paramtypes", [void 0, User]),
            __metadata("design:returntype", Promise)
        ], OnGetUser.prototype, "onGetUser", null);
        OnGetUser = __decorate([
            inversify_1.injectable()
            // @ts-ignore
            ,
            __param(0, inversify_1.inject(Logger)),
            __metadata("design:paramtypes", [Logger])
        ], OnGetUser);
        const container = new inversify_1.Container();
        container.bind(Logger).toSelf().inSingletonScope();
        container.bind(UserRepository).toSelf().inSingletonScope();
        container.bind(GetUserCommand).toSelf().inSingletonScope();
        container.bind(OnGetUser).toSelf().inSingletonScope();
        const runner = new CommandRunner_1.CommandRunner({
            container,
        });
        const result = yield runner.run(GetUserCommand, undefined);
        expect(result).toBeInstanceOf(User);
        expect(log).toHaveBeenCalledTimes(1);
        expect(log).toHaveBeenCalledWith('Hello, John Doe');
    }));
    it('allows async containers', () => __awaiter(void 0, void 0, void 0, function* () {
        class MyCommand {
            handle(event) {
                return __awaiter(this, void 0, void 0, function* () {
                    return event;
                });
            }
            static create() {
                return __awaiter(this, void 0, void 0, function* () {
                    return new MyCommand();
                });
            }
        }
        const runner = new CommandRunner_1.CommandRunner({
            container: {
                get: (Cls) => __awaiter(void 0, void 0, void 0, function* () {
                    return Cls.create();
                }),
            },
        });
        const result = yield runner.run(MyCommand, { foo: 'bar' });
        expect(result).toEqual({ foo: 'bar' });
    }));
});
//# sourceMappingURL=CommandRunner.spec.js.map