"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.AfterCommand = exports.BeforeCommand = exports.Use = void 0;
const MetadataStorage_1 = require("../metadata/MetadataStorage");
function sortByPriority(arr) {
    arr.sort((a, b) => (a.priority < b.priority ? 1 : -1));
}
function addSubscriber(kind, HandlerClass, options = {}, SubscriberClass, propertyKey) {
    let subscribers;
    if (MetadataStorage_1.storage.metadata.subscribers[kind].has(HandlerClass)) {
        subscribers = MetadataStorage_1.storage.metadata.subscribers[kind].get(HandlerClass);
    }
    else {
        subscribers = [];
        MetadataStorage_1.storage.metadata.subscribers[kind].set(HandlerClass, subscribers);
    }
    subscribers.push({
        HandlerClass,
        SubscriberClass,
        method: propertyKey,
        priority: typeof options.priority === 'number' ? options.priority : 0,
    });
    sortByPriority(subscribers);
}
function Use(HandlerClass, options = {}) {
    return (target, propertyKey, _descriptor) => {
        if (propertyKey !== 'handle') {
            throw new Error('@Use() can only be used on Commands.');
        }
        addSubscriber('middleware', target.constructor, options, HandlerClass, 'use');
    };
}
exports.Use = Use;
function BeforeCommand(HandlerClass, options = {}) {
    return (target, propertyKey, _descriptor) => {
        addSubscriber('before', HandlerClass, options, target.constructor, propertyKey);
    };
}
exports.BeforeCommand = BeforeCommand;
function AfterCommand(HandlerClass, options = {}) {
    return (target, propertyKey, _descriptor) => {
        addSubscriber('after', HandlerClass, options, target.constructor, propertyKey);
    };
}
exports.AfterCommand = AfterCommand;
//# sourceMappingURL=index.js.map